@@ -1,5 +1,5 @@
 # -*- test-case-name: nevow.test.test_guard -*-
-# Copyright (c) 2004 Divmod.
+# Copyright (c) 2004-2008 Divmod.
 # See LICENSE for details.
 
 """
@@ -9,11 +9,11 @@ L{SessionWrapper}.
 """
 
 __metaclass__ = type
-__version__ = "$Revision: 1.5 $"[11:-2]
 
 import random
 import time
 import md5
+import StringIO
 
 from zope.interface import implements
 
@@ -219,10 +219,8 @@ class Forbidden(object):
                 "<body><h1>Forbidden</h1>Request was forbidden.</body></html>")
 
 class SessionWrapper:
-    """SessionWrapper
-
-
-
+    """
+    SessionWrapper
 
     The following class attributes can be modified on an instance
     of the class.
@@ -243,7 +241,6 @@ class SessionWrapper:
         closed before the session timeout, both the session
         and the cookie go away.
     """
-
     implements(inevow.IResource)
 
     sessionLifetime = 3600
@@ -274,9 +271,6 @@ class SessionWrapper:
 
     def renderHTTP(self, ctx):
         request = inevow.IRequest(ctx)
-        request.setupSession = lambda : self.createSession(ctx, segments=[])
-        # ctx.remember(self, ILoginManager)
-
         d = defer.maybeDeferred(self._delegate, ctx, [])
         def _cb((resource, segments), ctx):
             assert not segments
@@ -287,13 +281,11 @@ class SessionWrapper:
 
     def locateChild(self, ctx, segments):
         request = inevow.IRequest(ctx)
-        # ctx.remember(self, ILoginManager)
         path = segments[0]
         if self.useCookies:
             cookie = request.getCookie(self.cookieKey)
         else:
             cookie = ''
-        request.setupSession = lambda : self.createSession(ctx, segments)
 
         if path.startswith(SESSION_KEY):
             key = path[len(SESSION_KEY):]
@@ -357,8 +349,10 @@ class SessionWrapper:
         return rd, ()
 
     def createSession(self, ctx, segments):
-        """Create a new session for this request, and redirect back to the path
-        given by segments."""
+        """
+        Create a new session for this request, and redirect back to the path
+        given by segments.
+        """
 
         request = inevow.IRequest(ctx)
 
@@ -374,11 +368,11 @@ class SessionWrapper:
                 expires = None
             request.addCookie(self.cookieKey, newCookie,
                               path="/%s" % '/'.join(request.prepath),
-                              secure=secure, expires=expires)
+                              secure=secure, expires=expires,
+                              domain=self.cookieDomainForRequest(request))
         sz = self.sessions[newCookie] = self.sessionFactory(self, newCookie)
         sz.args = request.args
-        sz.fields = getattr(request, 'fields', {})
-        sz.content = request.content
+        sz.fields = request.fields
         sz.method = request.method
         sz.received_headers = request.received_headers
         sz.checkExpired()
@@ -393,6 +387,10 @@ class SessionWrapper:
             - log the user out from our portal (calling their logout callback),
               if they are logged in and accessing a logout URL
 
+            - Move the request parameters saved on the session, if there are
+              any, onto the request if a session just started or a login
+              just succeeded.
+
         @return:
 
             - if the user is already logged in: a 2-tuple of requestObject,
@@ -427,10 +425,14 @@ class SessionWrapper:
         if spoof and hasattr(session, 'args'):
             request.args = session.args
             request.fields = session.fields
-            request.content = session.content
+            request.content = StringIO.StringIO()
+            request.content.close()
             request.method = session.method
             request.received_headers = session.received_headers
 
+            del session.args, session.fields, session.method, session.received_headers
+
+
         if segments and segments[0] in (LOGIN_AVATAR, LOGOUT_AVATAR):
             authCommand = segments[0]
         else:
@@ -561,3 +563,15 @@ class SessionWrapper:
         error.trap(UnauthorizedLogin)
         return Forbidden(), ()
 
+
+    def cookieDomainForRequest(self, request):
+        """
+        Specify the domain restriction on the session cookie.
+
+        @param request: The request object in response to which a cookie is
+            being set.
+
+        @return: C{None} or a C{str} giving the domain restriction to set on
+            the cookie.
+        """
+        return None
