@@ -62,11 +62,6 @@ rfc1808_relative_link_tests = [
     ]
 
 
-class FR(FakeRequest):
-    def prePathURL(Self):
-        return theurl
-
-
 
 class _IncompatibleSignatureURL(url.URL):
     """
@@ -115,7 +110,10 @@ class TestURL(TestCase):
             self.assertEquals(test, result)
 
     def test_fromRequest(self):
-        urlpath = url.URL.fromRequest(FR())
+        request = FakeRequest(uri='/a/nice/path/?zot=23&zut',
+                              currentSegments=["a", "nice", "path", ""])
+        request.setHeader('host', 'www.foo.com:80')
+        urlpath = url.URL.fromRequest(request)
         self.assertEquals(theurl, str(urlpath))
 
     def test_fromContext(self):
@@ -138,6 +136,15 @@ class TestURL(TestCase):
         self.failUnlessEqual(urlpath, url.URL.fromString(theurl))
         self.failIfEqual(urlpath, url.URL.fromString('ftp://www.anotherinvaliddomain.com/foo/bar/baz/?zot=21&zut'))
 
+
+    def test_fragmentEquality(self):
+        """
+        An URL created with the empty string for a fragment compares equal
+        to an URL created with C{None} for a fragment.
+        """
+        self.assertEqual(url.URL(fragment=''), url.URL(fragment=None))
+
+
     def test_parent(self):
         urlpath = url.URL.fromString(theurl)
         self.assertEquals("http://www.foo.com:80/a/nice/?zot=23&zut",
@@ -619,6 +626,17 @@ class Serialization(TestCase):
     test_rfc1808.todo = 'Many of these fail miserably at the moment; often with a / where there shouldn\'t be'
 
 
+    def test_unicode(self):
+        """
+        L{URLSerializer} should provide basic IRI (RFC 3987) support by
+        encoding Unicode to UTF-8 before percent-encoding.
+        """
+        iri = u'http://localhost/expos\xe9?doppelg\xe4nger=Bryan O\u2019Sullivan#r\xe9sum\xe9'
+        uri = 'http://localhost/expos%C3%A9?doppelg%C3%A4nger=Bryan%20O%E2%80%99Sullivan#r%C3%A9sum%C3%A9'
+        self.assertEquals(flatten(url.URL.fromString(iri)), uri)
+
+
+
 class RedirectResource(TestCase):
     """Test the url redirect resource adapters.
     """
