@@ -19,14 +19,16 @@ package_data={
         'nevow': [
             'Canvas.swf',
             '*.css',
-	    '*.js',
-	    'js/Divmod/*.js',
-	    'js/Nevow/*.js',
-	    'js/Nevow/Test/*.js',
-	    'js/Nevow/Athena/Tests/*.js',
-	    'js/Divmod/Runtime/*.js',
-	    'js/Nevow/Athena/*.js',
-	    'js/Nevow/TagLibrary/*.js',
+            '*.js',
+            'js/Divmod/*.js',
+            'js/Nevow/*.js',
+            'js/Nevow/Test/*.js',
+            'js/Nevow/Athena/Tests/*.js',
+            'js/Divmod/Runtime/*.js',
+            'js/Nevow/Athena/*.js',
+            'js/Nevow/TagLibrary/*.js',
+            'js/Divmod/Test/*.js',
+            'js/PythonTestSupport/*.js',
             ],
         'nevow.athena_private': [
             '*.png'
